// Example sketch showing how to create a node thay repeates messages
// from nodes far from gateway back to gateway. 
// It is important that nodes that has enabled repeater mode calls  
// gw.preocess() frequently. This node should never sleep. 

#include <MySensor.h>
#include <SPI.h>
#include <EEPROM.h>

#define DIGITAL_INPUT_SENSOR 3                  // The digital input you attached your sensor.  (Only 2 and 3 generates interrupt!)
#define PULSE_FACTOR 450                        // Nummber of blinks per m3 of your meter (One rotation/liter)
#define SLEEP_MODE false                        // flowvalue can only be reported when sleep mode is false.
#define MAX_FLOW 40                             // Max flow (l/min) value to report. This filetrs outliers.
#define INTERRUPT DIGITAL_INPUT_SENSOR-2        // Usually the interrupt = pin -2 (on uno/nano anyway)

unsigned long SEND_FREQUENCY = 30000; // Minimum time between send (in milliseconds). We don't wnat to spam the gateway. 

#define CHILD_ID     1                          // Id of the sensor child
#define CHILD_ID_DEV 254

MySensor gw;

double ppl = ((double)PULSE_FACTOR)/1000;        // Pulses per liter

volatile unsigned long pulseCount = 0;
volatile unsigned long lastBlink = 0;
volatile double flow = 0;
boolean pcReceived = false;
unsigned long oldPulseCount = 0;
unsigned long newBlink = 0;   
double oldflow = 0;
double volume;                     
double oldvolume;
unsigned long lastSend;
unsigned long lastPulse;
unsigned long currentTime; 

MyMessage flowMsg(CHILD_ID,V_FLOW);
MyMessage volumeMsg(CHILD_ID,V_VOLUME);
MyMessage pcMsg(CHILD_ID,V_VAR1);
MyMessage pcReqMsg(CHILD_ID,V_VAR2);

MyMessage PongMsg(CHILD_ID_DEV, V_SCENE_ON);

void setup()  
{  
  wdt_disable();

  // Init adress from pin
  if (analogRead(3) >= 1023){
    Serial.begin(115200);
    Serial.println("Init adress");
    
    for (int i=0;i<512;i++) {
      EEPROM.write(i, 0xff);
    } 
  } 
  
  // Disable the ADC by setting the ADEN bit (bit 7) to zero.
  ADCSRA = ADCSRA & B01111111;
  
  // Disable the analog comparator by setting the ACD bit
  // (bit 7) to one.
  ACSR = B10000000;  

  // The third argument enables repeater mode.
  gw.begin(incomingMessage, 27, false);
  
  flowMsg.setDestination(0);
  volumeMsg.setDestination(0);
  pcMsg.setDestination(0);
  pcReqMsg.setDestination(0);
  PongMsg.setDestination(0);

  //Send the sensor node sketch version information to the gateway
  gw.sendSketchInfo("Bath Room Sensor", "1.0");

  // Register this device as Waterflow sensor
  gw.present(CHILD_ID, S_WATER);       

  // Fetch last known pulse count value from gw
  //gw.request(CHILD_ID, V_VAR1);  
  gw.send(pcReqMsg.set("REQ"));

  attachInterrupt(INTERRUPT, onPulse, RISING); 
  
  wdt_enable(WDTO_8S);  
  
  lastSend=millis();  
}

void loop() 
{
  wdt_reset();
  
  // By calling process() you route messages in the background
  gw.process();

  // Send feq
  // Only send values at a maximum frequency or woken up from sleep
  currentTime = millis();
  bool sendTime = currentTime - lastSend > SEND_FREQUENCY;
 
  if (pcReceived && (SLEEP_MODE || sendTime)) {
    gw.send(PongMsg.set("PONG"));
    
    if (pcReceived) { 
      // New flow value has been calculated  
      if (!SLEEP_MODE && flow != oldflow) {
        // Check that we dont get unresonable large flow value. 
        // could hapen when long wraps or false interrupt triggered
        if (flow<((unsigned long)MAX_FLOW)) {
          gw.send(flowMsg.set(flow, 2));                   // Send flow value to gw
        }  
        //Serial.print("l/min:");
        //Serial.println(flow);
        oldflow = flow;
      }
    
      // No Pulse count in 2min 
      if(currentTime - lastPulse > 120000){
        flow = 0;
      } 
    
      // Pulse count has changed
      if (pulseCount != oldPulseCount) {
        gw.send(pcMsg.set(pulseCount));                // Send  volumevalue to gw VAR1
        double volume = ((double)pulseCount/((double)PULSE_FACTOR));     
        oldPulseCount = pulseCount;
        if (volume != oldvolume) {
          gw.send(volumeMsg.set(volume, 3));               // Send volume value to gw
          oldvolume = volume;
        } 
      } 
      
    } else if (sendTime && !pcReceived) {
      // No count received. Try requesting it again
      //gw.request(CHILD_ID, V_VAR1);      
      gw.send(pcReqMsg.set("REQ"));
    }
    
    lastSend=currentTime;
  }
  
  if (SLEEP_MODE && pcReceived) {
    gw.sleep(SEND_FREQUENCY);
  }  
}

void incomingMessage(const MyMessage &message) {
  if (message.type==V_VAR1) {
    pulseCount = oldPulseCount = message.getLong();
    Serial.print("Received last pulse count from gw:");
    Serial.println(pulseCount);
    pcReceived = true;
  }
}

void onPulse()     
{ 
  if (!SLEEP_MODE) {
    unsigned long newBlink = micros();   
    unsigned long interval = newBlink-lastBlink;
	lastPulse = millis();
    if (interval<500000L) {
      // Sometimes we get interrupt on RISING,  500000 = 0.5sek debounce ( max 120 l/min)
      return;   
    }
    flow = (60000000.0 /interval) / ppl;
    lastBlink = newBlink;
  }
  
  pulseCount++;  
}
